@@ -18,20 +18,22 @@
 #define _HAVE_COMBOBOX_H_
 
 /* KDE INCLUDES */
+#include <kcombobox.h>
 
 /* QT INCLUDES */
 #include <qstringlist.h>
 #include <qobject.h>
 #include <qstring.h>
+//Added by qt3to4:
+#include <QShowEvent>
 
 /* OTHER INCLUDES */
-#include <kcombobox.h>
 #include <kommanderwidget.h>
-#include <kommander_export.h>
+#include "kommanderwidgets_export.h"
 class QWidget;
 
 class QShowEvent;
-class KOMMANDER_EXPORT ComboBox : public KComboBox, public KommanderWidget
+class KOMMANDERWIDGETS_EXPORT ComboBox : public KComboBox, public KommanderWidget
 {
   Q_OBJECT
 
@@ -50,7 +52,8 @@ public:
   virtual QString populationText() const;
   virtual void setPopulationText(const QString&);
 
-  virtual QString handleDCOP(int function, const QStringList& args);
+  virtual QString handleDBUS(int function, const QStringList& args);
+  using KommanderWidget::handleDBUS;
   virtual bool isFunctionSupported(int function);
 public slots:
   virtual void setWidgetText(const QString&);
