@@ -1,20 +1,25 @@
 
-/***************************************************************************
-                          xsldbg.c  -  description
-                             -------------------
-    begin                : Sun Sep 16 2001
-    copyright            : (C) 2001 by Keith Isdale
-    email                : k_isdale@tpg.com.au
- ***************************************************************************/
+/**
+ *
+ *  This file is part of the kdewebdev package
+ *  Copyright (c) 2001 Keith Isdale <keith@kdewebdev.org>
+ *
+ *  This library is free software; you can redistribute it and/or 
+ *  modify it under the terms of the GNU General Public License as 
+ *  published by the Free Software Foundation; either version 2 of 
+ *  the License, or (at your option) any later version.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Library General Public License
+ *  along with this library; see the file COPYING.LIB.  If not, write to
+ *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA 02110-1301, USA.
+ **/
 
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
 
 /*
  * Based on file xsltproc.c
@@ -27,18 +32,21 @@
  *
  */
 
+#define WITH_XSLDBG_DEBUG_PROCESS
 /* Justin's port version - do not merge into core!!! */
 #define RISCOS_PORT_VERSION "2.01"
 
+#include "xsldbg.h"
+#include <QDateTime>
 #include <kurl.h>
 #include <kdebug.h>
-#include "xsldbg.h"
 #include "debug.h"
 #include "options.h"
 #include "utils.h"
 #include "files.h"
 #include "breakpoint.h"
 #include "debugXSL.h"
+#include "version.h"
 
 #include <libxml/xmlerror.h>
 #include "xsldbgmsg.h"
@@ -62,9 +70,6 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -74,9 +79,6 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#ifdef HAVE_STDARG_H
-#include <stdarg.h>
-#endif
 #include <libxml/xmlmemory.h>
 #include <libxml/debugXML.h>
 #include <libxml/xmlerror.h>
@@ -90,10 +92,8 @@
 #endif
 
 #include <libxml/catalog.h>
-#include <libxml/parserInternals.h>
 
 #include <libxslt/xslt.h>
-#include <libxslt/xsltInternals.h>
 #include <libxslt/transform.h>
 #include <libxslt/xsltutils.h>
 #include <libxslt/extensions.h>
@@ -101,7 +101,8 @@
 
 #include <kcmdlineargs.h> 
 #include <kglobal.h>
-#include <qfile.h>
+#include <kcomponentdata.h>
+#include <QFile>
 
 #ifdef WIN32
 #ifdef _MSC_VER
@@ -157,61 +158,54 @@ xmlParserInputPtr xmlNoNetExternalEntityLoader(const char *URL,
  -------------------------------------------*/
 
 /**
- * xsldbgInit:
+ * Initialize xsldbg:
  * 
- * Returns 1 if able to allocate memory needed by xsldbg
+ * @returns 1 if able to allocate memory needed by xsldbg
  *         0 otherwise
  */
 int xsldbgInit(void);
 
 
 /**
- * xsldbgFree:
- *
  * Free memory used by xsldbg
  */
 void xsldbgFree(void);
 
 
 /**
- * printTemplates:
- * @style : valid as parsed my xsldbg
- * @doc :    "    "   "     "    "
- *  
- * print out list of template names
+ * Pint out list of template names
+ *
+ * @param style : valid as parsed my xsldbg
+ * @param doc :    "    "   "     "    "
  */
 void printTemplates(xsltStylesheetPtr style, xmlDocPtr doc);
 
 
 /**
- * catchSigInt:
- * @value : not used
- *
  * Recover from a signal(SIGINT), exit if needed
+ *
+ * @param value : not used
  */
 void catchSigInt(int value);
 
 
 /**
- * catchSigTerm:
- * @value : not used
- *
  * Clean up and exit
+ *
+ * @param value : not used
  */
-void
-  catchSigTerm(int value);
+void   catchSigTerm(int value);
 
 /**
- * xsldbgGenericErrorFunc:
- * @ctx:  Is Valid
- * @msg:  Is valid
- * @...:  other parameters to use
- * 
  * Handles print output from xsldbg and passes it to the application if 
  *  running as a thread otherwise send to stderr
+ *
+ * @param ctx:  Is Valid
+ * @param msg:  Is valid
+ * ...:  other parameters to use
+ * 
  */
-void
-  xsldbgGenericErrorFunc(void *ctx, const char *msg, ...);
+void   xsldbgGenericErrorFunc(void *ctx, const char *msg, ...);
 
 xmlEntityPtr (*oldGetEntity)( void * user_data, const xmlChar * name);
 
@@ -236,16 +230,16 @@ static xmlEntityPtr xsldbgGetEntity( void * user_data, const xmlChar * name)
  * function calls
  */
 
-#if defined(HAVE_GETTIMEOFDAY)
-static struct timeval begin, end;
+
+static QTime startTime;
+
 
 /*
  * startTimer: call where you want to start timing
  */
-static void
-startTimer(void)
+static void startTimer(void)
 {
-    gettimeofday(&begin, NULL);
+    startTime = QDateTime::currentDateTime().time();
 }
 
 /*
@@ -253,102 +247,43 @@ startTimer(void)
  *           message about the timing performed; format is a printf
  *           type argument
  */
-static void
-endTimer(const QString& message)
+static void endTimer(const QString& message)
 {
-    long msec;
+    QTime endTime = QDateTime::currentDateTime().time();
+    int msec = startTime.msecsTo(endTime);
 
-    gettimeofday(&end, NULL);
-    msec = end.tv_sec - begin.tv_sec;
-    msec *= 1000;
-    msec += (end.tv_usec - begin.tv_usec) / 1000;
-
-#ifndef HAVE_STDARG_H
-#error "endTimer required stdarg functions"
-#endif
     /* Display the time taken to complete this task */
-    xsldbgGenericErrorFunc(i18n("%1 took %2 ms to complete.\n").arg(message).arg(msec));
+    xsldbgGenericErrorFunc(i18n("%1 took %2 ms to complete.\n", message, msec));
 }
-#elif defined(HAVE_TIME_H)
 
-/*
- * No gettimeofday function, so we have to make do with calling clock.
- * This is obviously less accurate, but there's little we can do about
- * that.
- */
 
-clock_t begin, end;
-static void
-startTimer(void)
-{
-    begin = clock();
-}
-static void
-endTimer(const QString& message)
-{
-    long msec;
-
-    end = clock();
-    msec = ((end - begin) * 1000) / CLOCKS_PER_SEC;
-
-#ifndef HAVE_STDARG_H
-#error "endTimer required stdarg functions"
-#endif
-    /* Display the time taken to complete this task */
-    xsldbgGenericErrorFunc(i18n("%1 took %2 ms to complete.\n").arg(message).arg(msec));
-}
-#else
-
-/*
- * We don't have a gettimeofday or time.h, so we just don't do timing
- */
-static void
-startTimer(void)
-{
-    /*
-     * Do nothing
-     */
-}
-static void
-endTimer(const QString& message)
-{
-    /*
-     * We can not do anything because we don't have a timing function
-     */
-#ifdef HAVE_STDARG_H
-    /* Display the time taken to complete this task */
-    xsldbgGenericErrorFunc(i18n("%1 took %2 ms to complete.\n"),arg(message).arg(msec));
-#else
-    /* We don't have gettimeofday, time or stdarg.h, what crazy world is
-     * this ?!
-     */
-#endif
-}
-#endif
-
-static void
-xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur)
+static void xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur)
 {
 
     xmlDocPtr res;
-    const char *params[8 * 2 + 2];
+    const char *params[(MAXPARAM_COUNT +1) * 2];
     int bytesWritten = -1;
     int nbparams = 0;
-    int paramIndex;
-    parameterItemPtr paramItem;
+    QByteArray paramArray[(MAXPARAM_COUNT +1) *2];
 
-    /* Copy the parameters accross for libxslt */
-    for (paramIndex = 0;
-         paramIndex < arrayListCount(optionsGetParamItemList());
-         paramIndex++) {
-        paramItem = (parameterItemPtr)arrayListGet(optionsGetParamItemList(), paramIndex);
-        if (paramItem) {
-            params[nbparams] = (char *) paramItem->name;
-            params[nbparams + 1] = (char *) paramItem->value;
+    Q_ASSERT(optionDataModel());
+    /* Copy the parameters across for libxslt */
+    QStringList paramsList = optionDataModel()->settingsList(XsldbgSettingsModel::ParamSettingType);
+    QString param;
+    XsldbgSettingData item;
+    foreach (param, paramsList) {
+        if (nbparams >= MAXPARAM_COUNT){
+            xsldbgGenericErrorFunc(i18n("Warning: Too many libxslt parameters provided.\n"));
+            break;
+        }
+        if (optionDataModel()->findSetting(param, XsldbgSettingsModel::ParamSettingType,  item)){
+            paramArray[nbparams].append(item.m_name.toUtf8().constData());
+            paramArray[nbparams + 1].append(item.m_value.toString().toUtf8().constData());
+            params[nbparams] = paramArray[nbparams].constData();
+            params[nbparams + 1] = paramArray[nbparams +1].constData(); 
             nbparams += 2;
         }
     }
-
     params[nbparams] = NULL;
 
 #ifdef LIBXML_XINCLUDE_ENABLED
@@ -358,7 +293,7 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur)
         xmlXIncludeProcess(doc);
         if (optionsGetIntOption(OPTIONS_TIMING)) {
 	    /* Display the time taken to do XInclude processing */
-            endTimer(i18n("XInclude processing %1.").arg((const char*)optionsGetStringOption(OPTIONS_DATA_FILE_NAME)));
+            endTimer(i18n("XInclude processing %1.", optionsGetStringOption(OPTIONS_DATA_FILE_NAME)));
         }
     }
 #endif
@@ -396,7 +331,7 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur)
                     notifyXsldbgApp(XSLDBG_MSG_FILEOUT,
                                     filesTempFileName(1));
                 } else {
-		     xsldbgGenericErrorFunc(i18n("Error: Unable to write temporary results to %1.\n").arg(filesTempFileName(1)));
+		     xsldbgGenericErrorFunc(i18n("Error: Unable to write temporary results to %1.\n", filesTempFileName(1)));
                     res = xsltProfileStylesheet(cur, doc, params, stderr);
                 }
             }
@@ -406,7 +341,7 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur)
         if (optionsGetIntOption(OPTIONS_PROFILING)) {
             if (optionsGetIntOption(OPTIONS_REPEAT))
 		/* Display how long it took to apply stylesheet */
-                endTimer(i18n("Applying stylesheet %n time", "Applying stylesheet %n times", optionsGetIntOption(OPTIONS_REPEAT)));
+                endTimer(i18np("Applying stylesheet %1 time", "Applying stylesheet %1 times", optionsGetIntOption(OPTIONS_REPEAT)));
             else
 		/* Display how long it took to apply stylesheet */
                 endTimer(i18n("Applying stylesheet"));
@@ -414,8 +349,7 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur)
         if (res == NULL) {
 #ifdef WITH_XSLDBG_DEBUG_PROCESS
             xsltGenericError(xsltGenericErrorContext, "Error: Transformation did not complete writing to file %s\n",
-                             optionsGetStringOption
-                             (OPTIONS_OUTPUT_FILE_NAME));
+                             optionsGetStringOption(OPTIONS_OUTPUT_FILE_NAME).toUtf8().constData());
 #endif
             return;
         }
@@ -443,7 +377,7 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur)
 			notifyXsldbgApp(XSLDBG_MSG_FILEOUT,
 				filesTempFileName(1));
 		    } else {
-			 xsldbgGenericErrorFunc(i18n("Error: Unable to write temporary results to %1.\n").arg(filesTempFileName(1)));
+			 xsldbgGenericErrorFunc(i18n("Error: Unable to write temporary results to %1.\n", filesTempFileName(1)));
 			xmlDebugDumpDocument(stdout, res);
 		    }
 
@@ -458,15 +392,13 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur)
 		    if (xslDebugStatus != DEBUG_QUIT) {
 			if (terminalIO != NULL)
 			    bytesWritten = xsltSaveResultToFile(terminalIO, res, cur);
-			else if (optionsGetStringOption
-				(OPTIONS_OUTPUT_FILE_NAME) == NULL)
+			else if (optionsGetStringOption(OPTIONS_OUTPUT_FILE_NAME).isEmpty())
 			    bytesWritten = xsltSaveResultToFile(stdout, res, cur);
 			else{
-			    bytesWritten = xsltSaveResultToFilename((const char *)
-				    optionsGetStringOption
-				    (OPTIONS_OUTPUT_FILE_NAME),
-				    res, cur, 0);
-		    }
+                            QByteArray outFile(optionsGetStringOption(OPTIONS_OUTPUT_FILE_NAME).toUtf8().constData());
+			    bytesWritten =
+                                xsltSaveResultToFilename(outFile.constData(), res, cur, 0);
+		        }
 		    }
 		    if (optionsGetIntOption(OPTIONS_TIMING))
 			/* Indicate how long it took to save to file */
@@ -476,21 +408,19 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur)
 			xsldbgGenericErrorFunc(i18n("Warning: Generating non-standard output XHTML.\n"));
 			if (optionsGetIntOption(OPTIONS_TIMING))
 			    startTimer();
-			if (terminalIO != NULL)
+			if (terminalIO != NULL){
 			    bytesWritten = xsltSaveResultToFile(terminalIO, res, cur);
-			else if (optionsGetStringOption
-				(OPTIONS_OUTPUT_FILE_NAME) == NULL)
+                        }else if (optionsGetStringOption(OPTIONS_OUTPUT_FILE_NAME).isEmpty()){
 			    bytesWritten = xsltSaveResultToFile(stdout, res, cur);
-			else
-			    bytesWritten = xsltSaveResultToFilename((const char *)
-				    optionsGetStringOption
-				    (OPTIONS_OUTPUT_FILE_NAME),
-				    res, cur, 0);
+                        }else{
+                            QByteArray outFile(optionsGetStringOption(OPTIONS_OUTPUT_FILE_NAME).toUtf8().constData());
+			    bytesWritten = xsltSaveResultToFilename(outFile.constData(), res, cur, 0);
+                        }
 			if (optionsGetIntOption(OPTIONS_TIMING))
 			    /* Indicate how long it took to save to file */
 			    endTimer(i18n("Saving result"));
 		    } else {
-			xsldbgGenericErrorFunc(i18n("Warning: Unsupported, non-standard output method %1.\n").arg(xsldbgText(cur->method)));
+			xsldbgGenericErrorFunc(i18n("Warning: Unsupported, non-standard output method %1.\n", xsldbgText(cur->method)));
 		    }
 		}
 	    }
@@ -502,9 +432,9 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur)
     } else {
         xsltTransformContextPtr userCtxt = xsltNewTransformContext(cur, doc);
         if (userCtxt){
-        bytesWritten = xsltRunStylesheetUser(cur, doc, params,
-                          (char *)optionsGetStringOption(OPTIONS_OUTPUT_FILE_NAME),
-                          NULL, NULL, NULL, userCtxt);
+            QByteArray outFile(optionsGetStringOption(OPTIONS_OUTPUT_FILE_NAME).toUtf8().constData());
+            bytesWritten = xsltRunStylesheetUser(cur, doc, params, outFile.constData(),
+                    NULL, NULL, NULL, userCtxt);
         if (optionsGetIntOption(OPTIONS_TIMING))
             endTimer(i18n("Running stylesheet and saving result"));
         xsltFreeTransformContext(userCtxt);
@@ -513,17 +443,17 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur)
         }
     }
     if ((xslDebugStatus != DEBUG_RUN_RESTART) && (bytesWritten == -1))
-	xsldbgGenericErrorFunc(i18n("Error: Unable to save results of transformation to file %1.\n").arg(xsldbgText(optionsGetStringOption(OPTIONS_OUTPUT_FILE_NAME)))); 
+	xsldbgGenericErrorFunc(i18n("Error: Unable to save results of transformation to file %1.\n", optionsGetStringOption(OPTIONS_OUTPUT_FILE_NAME))); 
+
 }
 
-int
-xsldbgMain(int argc, char **argv)
+int xsldbgMain(int argc, char **argv)
 {
     Q_UNUSED(argc);
     Q_UNUSED(argv);
     int i=0, result = 1, noFilesFound = 0;
     xsltStylesheetPtr cur = NULL;
-    xmlChar *expandedName;      /* contains file name with path expansion if any */
+    QString expandedName;      /* contains file name with path expansion if any */
 
     /* in some cases we always want to bring up a command prompt */
     int showPrompt;
@@ -539,9 +469,10 @@ xsldbgMain(int argc, char **argv)
     
 
     if (args){
-	QString langChoice = args->getOption("lang");
-	if (KGlobal::locale() && !langChoice.isEmpty() && result)
-	    KGlobal::locale()->setLanguage(langChoice);
+	    QString langChoice = args->getOption("lang");
+        if (KGlobal::locale() && !langChoice.isEmpty() && result) {
+            KGlobal::setLocale(new KLocale(KGlobal::mainComponent().catalogName(), langChoice));
+        }
     }
 
 #ifdef __riscos
@@ -555,6 +486,7 @@ xsldbgMain(int argc, char **argv)
     LIBXML_TEST_VERSION xmlLineNumbersDefault(1);
 
     if (!xsldbgInit()) {
+        qWarning("Init failed");
 	   xsldbgGenericErrorFunc(i18n("Fatal error: Aborting debugger due to an unrecoverable error.\n"));
         xsldbgFree();
         xsltCleanupGlobals();
@@ -570,20 +502,19 @@ xsldbgMain(int argc, char **argv)
 		break;
 
 	    if (args->arg(i)[0] != '-') {
-		expandedName = filesExpandName((const xmlChar*)args->arg(i));
-		if (!expandedName) {
+		expandedName = filesExpandName(args->arg(i));
+		if (expandedName.isEmpty()) {
+                    xsldbgGenericErrorFunc(i18n("Error: Unable to determine expanded file name for %1\n", (args->arg(i))));
 		    result = 0;
 		    break;
 		}
 		switch (noFilesFound) {
 		    case 0:
-			optionsSetStringOption(OPTIONS_SOURCE_FILE_NAME,
-				expandedName);
+			optionsSetStringOption(OPTIONS_SOURCE_FILE_NAME, expandedName);
 			noFilesFound++;
 			break;
 		    case 1:
-			optionsSetStringOption(OPTIONS_DATA_FILE_NAME,
-				expandedName);
+			optionsSetStringOption(OPTIONS_DATA_FILE_NAME, expandedName);
 			noFilesFound++;
 			break;
 
@@ -591,7 +522,6 @@ xsldbgMain(int argc, char **argv)
 			xsldbgGenericErrorFunc(i18n("Error: Too many file names supplied via command line.\n"));
 			result = 0;
 		}
-		xmlFree(expandedName);
 		continue;
 	    }
 	}  	
@@ -599,24 +529,27 @@ xsldbgMain(int argc, char **argv)
 
 	 // Handle boolean options
 	for (int optionID = OPTIONS_FIRST_BOOL_OPTIONID; optionID < OPTIONS_LAST_BOOL_OPTIONID; optionID++){
-	    if (optionsGetOptionName(OptionTypeEnum(optionID))){
-		if (args->isSet((char *)optionsGetOptionName(OptionTypeEnum(optionID))))
-		    optionsSetIntOption(OptionTypeEnum
-			    (optionID), 1);
+            QString optionName = optionsGetOptionName(OptionTypeEnum(optionID));
+	    if (!optionName.isEmpty()){
+		if (args->isSet(optionName.toUtf8().constData()))
+		    optionsSetIntOption(OptionTypeEnum(optionID), 1);
 		else
 		    optionsSetIntOption(OptionTypeEnum(optionID), 0);
 	    }
 	}
 
-	// No extra arguments go straight to the shell? 
-	if (args->count() == 0)
-	    result = optionsSetIntOption(OPTIONS_SHELL, 1);
+        if (args->isSet("autoloadconfig") && optionsAutoConfig()){
+            xmlChar *profile=0;
+            xsldbgReadConfig(profile);
+        }
 
-	// Disable net access? 
-	if (!args->isSet("net")){
-	    char noNetCmd[] = {"nonet 1"}; 
-	    xslDbgShellSetOption((xmlChar*)noNetCmd);
-	}
+        // No extra arguments go straight to the shell
+        if (args->count() == 0)
+            result = optionsSetIntOption(OPTIONS_SHELL, 1);
+    }else {
+        // nothing to do here as caller of xsldbgMain must setup the options required
+        // go straight to the shell 
+        result = optionsSetIntOption(OPTIONS_SHELL, 1);
     }
    
     if (getThreadStatus() != XSLDBG_MSG_THREAD_NOTUSED){
@@ -632,7 +565,7 @@ xsldbgMain(int argc, char **argv)
         xslDebugStatus = DEBUG_NONE;
     } else {
         xslDebugStatus = DEBUG_STOP;
-        xsltGenericError(xsltGenericErrorContext, "XSLDBG %s\n", VERSION);
+        xsltGenericError(xsltGenericErrorContext, "XSLDBG %s\n", KXSLDBG_VERSION);
     }
 
     if (optionsGetIntOption(OPTIONS_VALID))
@@ -645,11 +578,11 @@ xsldbgMain(int argc, char **argv)
 	if (args->isSet("verbose") && result)
 	    xsltSetGenericDebugFunc(stderr, NULL);
 
-	QCString outFile = args->getOption("output");
+	QString outFile = args->getOption("output").toUtf8();
 	if (!outFile.isEmpty() && result)
-	    result = xslDbgShellOutput((const xmlChar *)QFile::encodeName(outFile).data());
+	    result = xslDbgShellOutput((const xmlChar *)QFile::encodeName(outFile).constData());
 
-	QCString maxDepth = args->getOption("maxdepth");
+	QString maxDepth = args->getOption("maxdepth").toUtf8();
 	if (!maxDepth.isEmpty() && result){
 	    bool OK;
 	    int value = maxDepth.toInt(&OK);
@@ -664,38 +597,45 @@ xsldbgMain(int argc, char **argv)
 		optionsSetIntOption(OPTIONS_REPEAT, 100);
 	}
 
-	QCStringList xslParams(args->getOptionList("param"));
-	QCStringList::iterator it;
-	QCString param, paramName, paramValue;
-	int separatorIdx;
-	bool paramOK;
-	for ( it = xslParams.begin(); it != xslParams.end(); ++it){
-	    param = (*it);
-	    paramOK = true;
-	    separatorIdx = param.find(':');
-	    if (separatorIdx > 0){
-		paramName = param.left((uint)separatorIdx);
-		paramValue = param.mid((uint)separatorIdx + 1);
-		if (!paramName.isEmpty() && !paramValue.isEmpty()){
-		    if (arrayListCount(optionsGetParamItemList()) <= 31) {
-			arrayListAdd(optionsGetParamItemList(), optionsParamItemNew((const xmlChar*)paramName.data(), (const xmlChar*)paramValue.data()));
-		    }else{
-			xsldbgGenericErrorFunc(i18n("Warning: Too many libxslt parameters provided via the command line option --param.\n"));
-		    }
-		}else{
-		    paramOK = false;
-		}
-	    }else{
-		paramOK = false;
-	    }
-	    if (!paramOK)
-		xsldbgGenericErrorFunc(i18n("Error: Argument \"%1\" to --param is not in the format <name>:<value>.\n").arg(param.data()));
+        QStringList xslParams(args->getOptionList("param"));
+        QStringList::iterator it;
+        QString param;
+        QString paramName;
+        QVariant paramValue(QVariant::String);
+        int separatorIdx;
+        int paramCount = 0;
+        bool paramOK;
+        for ( it = xslParams.begin(); it != xslParams.end(); ++it){
+            param = (*it);
+            paramOK = true;
+            separatorIdx = param.indexOf(':');
+            // if a ':' is not found try '='
+            if (separatorIdx <= 0)
+                separatorIdx = param.indexOf('=');
+            if (separatorIdx > 0){
+                paramName = param.left((uint)separatorIdx);
+                paramValue = param.mid((uint)separatorIdx + 1);
+                if (!paramName.isEmpty() && paramValue.isValid() && optionDataModel()){
+                    if (paramCount <= MAXPARAM_COUNT)
+                        paramOK = optionDataModel()->addParameter(paramName, paramValue);
+                    else
+                        paramOK = false;
+
+                    if (!paramOK)
+                        xsldbgGenericErrorFunc(i18n("Warning: Unable to add libxslt parameter %1.\n", param));
+                }else{
+                    xsldbgGenericErrorFunc(i18n("Error: Argument \"%1\" to --param is not in the format <name>:<value>.\n", param));
+                    paramOK = false;
+                }
+            }else{
+                xsldbgGenericErrorFunc(i18n("Error: Argument \"%1\" to --param is not in the format <name>:<value>.\n", param));
+                paramOK = false;
+            }
 	}
 
-	QCString cdPath = args->getOption("cd");
-	if (!cdPath.isEmpty() && result)
-	    result = changeDir((const xmlChar *)QFile::encodeName(cdPath).data());
-
+        // set the new CWD required, this will later result in a call to changeDir()
+        if (!args->getOption("cd").isEmpty())
+            optionsSetStringOption(OPTIONS_CWD, filesExpandName(args->getOption("cd"), false));
     }
 
     if (!result) {
@@ -705,19 +645,6 @@ xsldbgMain(int argc, char **argv)
     }
 
 
-    /*
-     * * Replace entities with their content.
-     */
-    xmlSubstituteEntitiesDefault(1);
-
-    /*
-     * * Register the EXSLT extensions and the test module
-     */
-    exsltRegisterAll();
-    xsltRegisterTestModule();
-
-    
-
     debugGotControl(0);
     while (xslDebugStatus != DEBUG_QUIT) {
 	xsldbgReachedFirstTemplate = false;
@@ -726,35 +653,51 @@ xsldbgMain(int argc, char **argv)
 	cur = NULL;
 	doc = NULL;
 	arrayListEmpty(filesEntityList());
-	xsltSetXIncludeDefault(optionsGetIntOption(OPTIONS_XINCLUDE));
-
 	/* copy the volitile options over to xsldbg */
 	optionsCopyVolitleOptions();
 
+        // apply the chosen working directory to this session
+        changeDir(optionsGetStringOption(OPTIONS_CWD));
+
+        // use xinclude and net entity loader selection
+        xsltSetXIncludeDefault(optionsGetIntOption(OPTIONS_XINCLUDE));
+        /* enable/disable the no net entity loader as required */
+        if (optionsGetIntOption(OPTIONS_NET)) 
+            xmlSetExternalEntityLoader(xmlNoNetExternalEntityLoader);
+        else
+            xmlSetExternalEntityLoader(xsldbgDefaultEntLoader);
+
 	/* choose where error messages/xsldbg output get sent to */
 	if (optionsGetIntOption(OPTIONS_STDOUT))
 	    errorFile = stdout;
 	else
 	    errorFile = stderr;
 
+        /* enable/disable the no net entity loader as required */
+        if (optionsGetIntOption(OPTIONS_NET)) 
+            xmlSetExternalEntityLoader(xmlNoNetExternalEntityLoader);
+        else
+            xmlSetExternalEntityLoader(xsldbgDefaultEntLoader);
+
 	filesLoadCatalogs();
 
 	if (optionsGetIntOption(OPTIONS_SHELL)) {
 	    debugGotControl(0);
+
 	    xsldbgGenericErrorFunc(i18n("\nStarting stylesheet\n\n"));
 	    if (optionsGetIntOption(OPTIONS_TRACE) == TRACE_OFF)
 		xslDebugStatus = DEBUG_STOP;    /* stop as soon as possible */
 	}
 
-	if ((optionsGetStringOption(OPTIONS_SOURCE_FILE_NAME) == NULL) ||
-		(optionsGetStringOption(OPTIONS_DATA_FILE_NAME) == NULL)) {
+	if (optionsGetStringOption(OPTIONS_SOURCE_FILE_NAME).isEmpty() ||
+		optionsGetStringOption(OPTIONS_DATA_FILE_NAME).isEmpty()) {
 	    /* at least on file name has not been set */
 	    /*goto a xsldbg command prompt */
 	    showPrompt = 1;
-	    if (optionsGetStringOption(OPTIONS_SOURCE_FILE_NAME) == NULL)
+	    if (optionsGetStringOption(OPTIONS_SOURCE_FILE_NAME).isEmpty())
 		xsldbgGenericErrorFunc(i18n("Error: No XSLT source file supplied.\n"));
 
-	    if (optionsGetStringOption(OPTIONS_DATA_FILE_NAME) == NULL) {
+	    if (optionsGetStringOption(OPTIONS_DATA_FILE_NAME).isEmpty()) {
 		xsldbgGenericErrorFunc(i18n("Error: No XML data file supplied.\n"));
 	    }
 
@@ -797,7 +740,7 @@ xsldbgMain(int argc, char **argv)
 		    showPrompt = 1;
 		    xslDebugStatus = DEBUG_STOP;
 		} else {
-		    xsldbgGenericErrorFunc(i18n("\nFinished stylesheet\n\032\032\n"));
+		    xsldbgGenericErrorFunc(i18n("\nFinished stylesheet\n\n"));
 		    {
 			/* handle trace execution */
 			int trace = optionsGetIntOption(OPTIONS_TRACE);
@@ -834,7 +777,7 @@ xsldbgMain(int argc, char **argv)
 		xslDebugStatus = DEBUG_QUIT;
 	    }
 	} else {
-	    /* Some sort of problem loading source file has occured. Quit? */
+	    /* Some sort of problem loading source file has occurred. Quit? */
 	    if (xslDebugStatus == DEBUG_NONE) {
 		xslDebugStatus = DEBUG_QUIT;    /* Panic!! */
 		result = 0;
@@ -893,31 +836,19 @@ xsldbgMain(int argc, char **argv)
     return !result;
 }
 
-/**
- * xsldbgLoadStylesheet:
- *
- * Load the stylesheet and return it 
- *
- * Returns The stylesheet after reloading it if successful
- *         NULL otherwise
- */
-xsltStylesheetPtr
-xsldbgLoadStylesheet()
+xsltStylesheetPtr xsldbgLoadStylesheet()
 {
     xsltStylesheetPtr cur = NULL;
     xmlDocPtr style;
+    QByteArray sourceFile(optionsGetStringOption(OPTIONS_SOURCE_FILE_NAME).toUtf8().constData());
 
     if (optionsGetIntOption(OPTIONS_TIMING))
         startTimer();
-#if LIBXML_VERSION >= 2006000
-    style = xmlSAXParseFile(&mySAXhdlr, (const char *) optionsGetStringOption(OPTIONS_SOURCE_FILE_NAME), 0);
-#else
-    style = xmlParseFile((const char *) optionsGetStringOption(OPTIONS_SOURCE_FILE_NAME));
-#endif
+    style = xmlParseFile(sourceFile.constData());
     if (optionsGetIntOption(OPTIONS_TIMING))
-        endTimer(i18n("Parsing stylesheet %1").arg((const char*)optionsGetStringOption(OPTIONS_SOURCE_FILE_NAME)));
+        endTimer(i18n("Parsing stylesheet %1", sourceFile.constData()));
     if (style == NULL) {
-        xsldbgGenericErrorFunc(i18n("Error: Cannot parse file %1.\n").arg(xsldbgUrl(optionsGetStringOption(OPTIONS_SOURCE_FILE_NAME))));
+        xsldbgGenericErrorFunc(i18n("Error: Cannot parse file %1.\n", xsldbgUrl(sourceFile)));
         cur = NULL;
         if (!optionsGetIntOption(OPTIONS_SHELL)) {
             xsldbgGenericErrorFunc(i18n("Fatal error: Aborting debugger due to an unrecoverable error.\n"));
@@ -949,44 +880,37 @@ xsldbgLoadStylesheet()
 
 
 
-/**
- * xsldbgLoadXmlData:
- *
- * Load the xml data file and return it  
- *
- * Returns The data file after reloading it if successful
- *         NULL otherwise
- */
-xmlDocPtr
-xsldbgLoadXmlData(void)
+xmlDocPtr xsldbgLoadXmlData(void)
 {
     xmlDocPtr doc = NULL;
+    xmlSAXHandler mySAXHandler;
+    QByteArray dataFile(optionsGetStringOption(OPTIONS_DATA_FILE_NAME).toUtf8().constData());
     doc = NULL;
 
+    xmlSAXVersion(&mySAXHandler,2);
+    oldGetEntity = mySAXHandler.getEntity;
+    mySAXHandler.getEntity = xsldbgGetEntity;
+
     if (optionsGetIntOption(OPTIONS_TIMING))
         startTimer();
 #ifdef LIBXML_HTML_ENABLED
     if (optionsGetIntOption(OPTIONS_HTML))
-        doc = htmlParseFile((char *)
-                            optionsGetStringOption(OPTIONS_DATA_FILE_NAME),
-                            NULL);
+        doc = htmlParseFile(dataFile.constData(), NULL);
     else
 #endif
 #ifdef LIBXML_DOCB_ENABLED
     if (optionsGetIntOption(OPTIONS_DOCBOOK))
-        doc = docbParseFile((char *)
-                            optionsGetStringOption(OPTIONS_DATA_FILE_NAME),
-                            NULL);
+        doc = docbParseFile(dataFile.constData(), NULL);
     else
 #endif
-#if LIBXML_VERSION >= 2006000
-        doc = xmlSAXParseFile(&xmlDefaultSAXHandler,
-			     (char *) optionsGetStringOption(OPTIONS_DATA_FILE_NAME), 0);
+
+#if LIBXML_VERSION >= 20600
+        doc = xmlSAXParseFile(&mySAXHandler, dataFile.constData(), 0);
 #else
-        doc = xmlParseFile((char *) optionsGetStringOption(OPTIONS_DATA_FILE_NAME));
+        doc = xmlParseFile(dataFile.constData());
 #endif
     if (doc == NULL) {
-        xsldbgGenericErrorFunc(i18n("Error: Unable to parse file %1.\n").arg(xsldbgUrl(optionsGetStringOption(OPTIONS_DATA_FILE_NAME))));
+        xsldbgGenericErrorFunc(i18n("Error: Unable to parse file %1.\n", xsldbgUrl(dataFile)));
         if (!optionsGetIntOption(OPTIONS_SHELL)) {
             xsldbgGenericErrorFunc(i18n("Fatal error: Aborting debugger due to an unrecoverable error.\n"));
             xslDebugStatus = DEBUG_QUIT;
@@ -995,23 +919,13 @@ xsldbgLoadXmlData(void)
             xslDebugStatus = DEBUG_STOP;
         }
     } else if (optionsGetIntOption(OPTIONS_TIMING))
-        endTimer(QString("Parsing document %1").arg(xsldbgUrl(optionsGetStringOption(OPTIONS_DATA_FILE_NAME))).utf8().data());
+        endTimer(QString("Parsing document %1").arg(xsldbgUrl(dataFile)).toUtf8().constData());
 
     return doc;
 }
 
 
-/**
- * xsldbgLoadXmlTemporary:
- * @path: The name of temporary file to load 
- *
- * Load the temporary data file and return it 
- *
- * Returns The temporary file after reloading it if successful,
- *         NULL otherwise
- */
-xmlDocPtr
-xsldbgLoadXmlTemporary(const xmlChar * path)
+xmlDocPtr xsldbgLoadXmlTemporary(const xmlChar * path)
 {
     xmlDocPtr doc = NULL;
     doc = NULL;
@@ -1030,7 +944,7 @@ xsldbgLoadXmlTemporary(const xmlChar * path)
 #endif
         doc = xmlSAXParseFile(&mySAXhdlr, (char *) path, 0);
     if (doc == NULL) {
-        xsldbgGenericErrorFunc(i18n("Error: Unable to parse file %1.\n").arg(xsldbgUrl(path)));
+        xsldbgGenericErrorFunc(i18n("Error: Unable to parse file %1.\n", xsldbgUrl(path)));
     }
 
     if (optionsGetIntOption(OPTIONS_TIMING)
@@ -1040,15 +954,7 @@ xsldbgLoadXmlTemporary(const xmlChar * path)
     return doc;
 }
 
-/**
- * printTemplates:
- * @style : valid as parsed my xsldbg
- * @doc :    "    "   "     "    "
- *  
- * print out list of template names
- */
-void
-printTemplates(xsltStylesheetPtr style, xmlDocPtr doc)
+void printTemplates(xsltStylesheetPtr style, xmlDocPtr doc)
 {
     xsltTransformContextPtr ctxt = xsltNewTransformContext(style, doc);
 
@@ -1123,14 +1029,7 @@ void xsldbgSAXWarningHandler(void * ctx, const char * msg, ...)
 
 #endif
 
-/**
- * catchSigInt:
- * @value : not used
- *
- * Recover from a signal(SIGINT), exit if needed
- */
-void
-catchSigInt(int value)
+void catchSigInt(int value)
 {
     Q_UNUSED(value);
     if ((xslDebugStatus == DEBUG_NONE) || (xsldbgStop == 1) || (xslDebugStatus == DEBUG_STOP)) {
@@ -1149,14 +1048,7 @@ catchSigInt(int value)
 }
 
 
-/**
- * catchSigTerm:
- * @value : not used
- *
- * Clean up and exit
- */
-void
-catchSigTerm(int value)
+void catchSigTerm(int value)
 {
     Q_UNUSED(value);
     xsldbgFree();
@@ -1170,21 +1062,13 @@ static sighandler_t oldHandler;
 
 static int initialized = 0;
 
-/**
- * xsldbgInit:
- * 
- * Returns 1 if able to allocate memory needed by xsldbg
- *         0 otherwise
- */
-int
-xsldbgInit()
+int xsldbgInit()
 {
     int result = 0;
     int xmlVer = 0;
 
     if (!initialized) {
         sscanf(xmlParserVersion, "%d", &xmlVer);
-        xsldbgHasLineNumberFix = (xmlVer >= 20508);
         if (!debugInit()) {
 #ifdef WITH_XSLDBG_DEBUG_PROCESS
             xsltGenericError(xsltGenericErrorContext,
@@ -1221,7 +1105,7 @@ xsldbgInit()
         /* set up the parser */
         xmlInitParser();
 #if 0
-#if LIBXML_VERSION >= 2006000
+#if LIBXML_VERSION >= 20600
      xmlSetGenericErrorFunc(NULL,  NULL);
      xmlSetStructuredErrorFunc(NULL , (xmlStructuredErrorFunc)xsldbgStructErrorHandler);
 #else
@@ -1233,15 +1117,22 @@ xsldbgInit()
         xsltSetGenericErrorFunc(0, xsldbgGenericErrorFunc);
 #endif
 
+    /*
+     * * Replace entities with their content.
+     */
+    xmlSubstituteEntitiesDefault(1);
+
+    /*
+     * * Register the EXSLT extensions and the test module
+     */
+    exsltRegisterAll();
+    xsltRegisterTestModule();
+
 	/*
 	 * disable CDATA from being built in the document tree
 	 */
 	xmlDefaultSAXHandlerInit();
 	xmlDefaultSAXHandler.cdataBlock = NULL;
-if (xsldbgHasLineNumberFix){
-	oldGetEntity = xmlDefaultSAXHandler.getEntity;
-	xmlDefaultSAXHandler.getEntity = xsldbgGetEntity;
-}
 
         if (getThreadStatus() != XSLDBG_MSG_THREAD_NOTUSED) {
             initialized = 1;
@@ -1264,13 +1155,7 @@ if (xsldbgHasLineNumberFix){
     return 1;
 }
 
-/**
- * xsldbgFree:
- *
- * Free memory used by xsldbg
- */
-void
-xsldbgFree()
+void xsldbgFree()
 {
     debugFree();
     filesFree();
@@ -1297,17 +1182,7 @@ xsldbgFree()
 
 char msgBuffer[4000];
 
-/**
- * xsldbgGenericErrorFunc:
- * @ctx:  Is Valid
- * @msg:  Is valid
- * @...:  other parameters to use
- * 
- * Handles print output from xsldbg and passes it to the application if 
- *  running as a thread otherwise send to stderr
- */
-void
-xsldbgGenericErrorFunc(void *ctx, const char *msg, ...)
+void xsldbgGenericErrorFunc(void *ctx, const char *msg, ...)
 {
     va_list args;
     Q_UNUSED(ctx);
@@ -1333,23 +1208,13 @@ xsldbgGenericErrorFunc(void *ctx, const char *msg, ...)
 
 void xsldbgGenericErrorFunc(QString const &text)
 {
-    xsldbgGenericErrorFunc(0, "%s", text.utf8().data());
+    xsldbgGenericErrorFunc(0, "%s", text.toUtf8().constData());
 }
 
 
 QString xsldbgUrl(const char *utf8fUrl)
 {
-    QString tempUrl(utf8fUrl);
-    QString fixedURI;
-    KURL url(tempUrl);
-
-    // May be provided with a URL that only has been encoded and has no protocol prefix ie a local file
-    if ( !tempUrl.startsWith("file:/")  &&  !tempUrl.startsWith("http:/") && !tempUrl.startsWith("ftp:/"))
-	fixedURI = KURL::decode_string(tempUrl);
-    else
-	fixedURI = url.prettyURL();
-
-    return fixedURI;
+    return filesExpandName(xsldbgText(utf8fUrl));
 }
 
 QString xsldbgUrl(const xmlChar *utf8Url)
@@ -1360,7 +1225,6 @@ QString xsldbgUrl(const xmlChar *utf8Url)
 QString xsldbgText(const char *utf8Text)
 {
     return QString::fromUtf8(utf8Text);
-    
 }
 
 QString xsldbgText(const xmlChar *utf8Text)
